# This file is generated by wxPython's SIP generator.  Do not edit by hand.
#
# Copyright: (c) 2018 by Total Control Software
# License:   wxWindows License

"""
These classes enable viewing and interacting with an OpenGL context in a wx.Window.
"""

from ._glcanvas import *

import wx

