# This file is generated by wxPython's SIP generator.  Do not edit by hand.
#
# Copyright: (c) 2018 by Total Control Software
# License:   wxWindows License

"""
Some simple XML classes for use with XRC.

For more advanced XML needs it would be better to use one of the XML packages
provided by Python.
"""

from ._xml import *

import wx

XmlProperty = wx.deprecated(XmlAttribute, 'Use XmlProperty instead.')

