# This file was generated by wxPython's wscript.

VERSION_STRING    = '4.0.3'
MAJOR_VERSION     = 4
MINOR_VERSION     = 0
RELEASE_NUMBER    = 3
BUILD_TYPE        = 'development'

VERSION = (MAJOR_VERSION, MINOR_VERSION, RELEASE_NUMBER, '')
