# This file is generated by wxPython's SIP generator.  Do not edit by hand.
#
# Copyright: (c) 2018 by Total Control Software
# License:   wxWindows License

"""
The :ref:`wx.webkit.wxWebKitCtrl` and related classes are provided mainly for
backwards compatibility only. New code would be more future-proof by using the
``wx.html2`` module.  The classes in this module are light wrappers around
the OSX WebKit control and is not implemented on any other platform.
"""

from ._webkit import *

import wx

EVT_WEBKIT_BEFORE_LOAD = wx.PyEventBinder( wxEVT_WEBKIT_BEFORE_LOAD, 1 )
EVT_WEBKIT_STATE_CHANGED = wx.PyEventBinder( wxEVT_WEBKIT_STATE_CHANGED, 1 )
EVT_WEBKIT_NEW_WINDOW = wx.PyEventBinder( wxEVT_WEBKIT_NEW_WINDOW, 1 )

